#include "amg8833.h"
#include "i2c.h"

struct pctl _pctl;
struct rst _rst;
struct fpsc _fpsc;
struct intc _intc;
struct stat _stat;
struct sclr _sclr;
struct ave _ave;
struct inthl _inthl;
struct inthh _inthh;
struct intll _intll;
struct intlh _intlh;
struct ihysl _ihysl;
struct ihysh _ihysh;
struct tthl _tthl;
struct tthh _tthh;

uint8_t getPCTL(void){ return _pctl.PCTL; }
uint8_t getRST(void){	return _rst.RST; }
uint8_t getFPSC(void){ return _fpsc.FPS & 0x00; }
uint8_t getINTC(void){ return (_intc.INTMOD << 1 | _intc.INTEN) & 0x03; }
uint8_t getSTAT(void){ return ( (_stat.OVF_THS << 3) | (_stat.OVF_IRS << 2) | (_stat.INTF << 1) ) & 0x07; }
uint8_t getSCLR(void){ return ((_sclr.OVT_CLR << 3) | (_sclr.OVS_CLR << 2) | (_sclr.INTCLR << 1)) & 0x07; }
uint8_t getAVE(void){ return (_ave.MAMOD << 5); }
uint8_t getINTHL(void){ return _inthl.INT_LVL_H; }
uint8_t getINTHH(void){ return _inthh.INT_LVL_H; }
uint8_t getINTLL(void){ return _intll.INT_LVL_L; }
uint8_t getINTLH(void){ return (_intlh.INT_LVL_L & 0xF); }
uint8_t getIHYSL(void){ return _ihysl.INT_HYS; }
uint8_t getIHYSH(void){ return (_ihysh.INT_HYS & 0xF); }
uint8_t getTTHL(void){ return _tthl.TEMP; }
uint8_t getTTHH(void){ return ( (_tthh.SIGN << 3) | _tthh.TEMP) & 0xF; }			
uint8_t min(uint8_t a, uint8_t b){ return a < b ? a : b; }
#define constrain(amt,low,high) ((amt)<(low)?(low):((amt)>(high)?(high):(amt)))

//IICд
//addr:ַ 
//reg:Ĵַ
//len:д볤
//buf:
void write(uint8_t reg,uint8_t *buf,uint8_t len){
	uint8_t i,data[100],error;
	data[0]=reg;
	for(i=1;i<len+1;i++){
		data[i]=buf[i-1];
	}
	error=I2C_MasterWrite(MXC_I2C0, (AMG88xx_ADDRESS<<1)|0, data, len+1, 0);
	//printf("%X    %X    %d\n",data[0],data[1],error);
} 
//IIC
//addr:ַ
//reg:ҪȡļĴַ
//len:Ҫȡĳ
//buf:ȡݴ洢
void read(uint8_t reg,uint8_t *buf,uint8_t len){ 
	//uint8_t res;
	I2C_MasterWrite(MXC_I2C0, (AMG88xx_ADDRESS<<1)|0, &reg, 1, 0);	
	I2C_MasterRead(MXC_I2C0, (AMG88xx_ADDRESS<<1)|1, buf, len, 0);
	//res=I2C_MasterRead(MXC_I2C0, (AMG88xx_ADDRESS<<1)|1, buf, len, 0);
	//printf("read len=%d\n",res);	
}
void write8(uint8_t reg,uint8_t data) {
	write(reg, &data, 1);
}
uint8_t read8(uint8_t reg) {
  uint8_t ret;
	read(reg, &ret, 1);	
	return ret;
}

/**************************************************************************/
/*! 
    @brief  convert a 12-bit signed magnitude value to a floating point number
    @param  val the 12-bit signed magnitude value to be converted
    @returns the converted floating point value
*/
/**************************************************************************/
float signedMag12ToFloat(uint16_t val){
	//take first 11 bits as absolute val
	uint16_t absVal = (val & 0x7FF);	
	return (val & 0x8000) ? 0 - (float)absVal : (float)absVal ;
}


/**************************************************************************/
/*! 
    @brief  disable the interrupt pin on the device
*/
/**************************************************************************/
void disableInterrupt(){
	_intc.INTEN = 0;
	write8(AMG88xx_INTC, getINTC());
}
/**************************************************************************/
/*! 
    @brief  Setups the I2C interface and hardware
    @param  addr Optional I2C address the sensor can be found on. Default is 0x69
    @returns True if device is set up, false on any failure
*/
/**************************************************************************/
int amg88xxInit(void){
	//enter normal mode
	_pctl.PCTL = AMG88xx_NORMAL_MODE;
	write8(AMG88xx_PCTL, getPCTL());	
	//software reset
	_rst.RST = AMG88xx_INITIAL_RESET;
	write8(AMG88xx_RST, getRST());
	
	//disable interrupts by default
	disableInterrupt();
	
	//set to 10 FPS
	_fpsc.FPS = AMG88xx_FPS_10;
	write8(AMG88xx_FPSC, getFPSC());

	delay_ms(10);

	return 0;
}
//ȡ8*8=64¶ֵ
void readPixels(float* pixbuf){
	uint8_t bufraw[128],i,pos;
	uint16_t recast;
	read(AMG88xx_PIXEL_OFFSET, (uint8_t*)bufraw, 128);
	//128תΪ¶ȣ϶ȣ
	for(i=0;i<64;i++){
		pos = i << 1;
		recast = ((uint16_t)bufraw[pos + 1] << 8) | ((uint16_t)bufraw[pos]);        		
		pixbuf[i] = signedMag12ToFloat(recast) * AMG88xx_PIXEL_TEMP_CONVERSION;
	}	
}
//ȡοֵ¶
/**************************************************************************/
/*! 
    @brief  read the onboard thermistor
    @returns a the floating point temperature in degrees Celsius
*/
/**************************************************************************/
float readThermistor(){
	uint8_t raw[2];
	read(AMG88xx_TTHL, raw, 2);
	uint16_t recast = ((uint16_t)raw[1] << 8) | ((uint16_t)raw[0]);
	return signedMag12ToFloat(recast) * AMG88xx_THERMISTOR_CONVERSION;
}

